@@ -16,9 +16,13 @@ from .traced_finder import DEFAULT_MODULE_NAME, LIVE_MODULE_NAME, \
 
 try:
     from .mock_turtle import MockTurtle
-    from .live_image import LiveFigure, monkey_patch_pyglet
+    from .live_image import LiveFigure, LivePillowImage, monkey_patch_pyglet
 except ImportError:
-    MockTurtle = monkey_patch_pyglet = LiveFigure = None
+    MockTurtle = monkey_patch_pyglet = LiveFigure = LivePillowImage = None
+
+
+class SourceLoadError(IOError):
+    pass
 
 
 class DelegatingModuleFinder(MetaPathFinder):
@@ -151,8 +155,12 @@ class TracedModuleImporter(DelegatingModuleFinder, Loader):
                     self.source_code = source_file.read()
             source_code = self.source_code
         else:
-            with open(module_file) as source_file:
-                source_code = source_file.read()
+            try:
+                with open(module_file) as source_file:
+                    source_code = source_file.read()
+            except Exception as ex:
+                message = f'Cannot read Python source from {module_file}.'
+                raise SourceLoadError(message) from ex
         module_name = module.__name__
         is_module_traced = False
         source_tree = None
@@ -323,6 +331,8 @@ class TracedModuleImporter(DelegatingModuleFinder, Loader):
             elif traced.startswith(LIVE_MODULE_NAME):
                 traced = traced[len(LIVE_MODULE_NAME)+1:]
                 self.is_traced_module_imported = True
+            if self.is_traced_module_imported and self.source_code is None:
+                self.source_code = source
             parsed_file = PSEUDO_FILENAME if traced == '' else filename
             self.driver_finder = TracedFinder(source,
                                               traced,
@@ -375,7 +385,8 @@ class PatchedModuleFinder(DelegatingModuleFinder):
                             'matplotlib.pyplot',
                             'numpy.random',
                             'random',
-                            'pyglet'):
+                            'pyglet',
+                            'PIL.ImageShow'):
             return None
         spec = super(PatchedModuleFinder, self).find_spec(fullname, path, target)
         if spec is not None:
@@ -404,27 +415,47 @@ class PatchedModuleLoader(Loader):
             module.use('Agg')
         elif self.fullname == 'matplotlib.pyplot':
             self.plt = module
-            # noinspection PyProtectedMember
-            turtle_screen = MockTurtle._screen
-            screen_width = turtle_screen.cv.cget('width')
-            screen_height = turtle_screen.cv.cget('height')
-            module.show = partial(self.mock_show)  # Lets it accept a signature.
-            module.live_coding_size = (screen_width, screen_height)
-            module.live_coding_zoom = self.live_coding_zoom
+            module.show = partial(self.mock_show_matplotlib)  # Lets it accept a signature.
+            module.live_coding_size = self._screen_size()
+            module.live_coding_zoom = self.live_coding_zoom_matplotlib
             if self.is_zoomed:
-                self.live_coding_zoom()
+                self.live_coding_zoom_matplotlib()
         elif self.fullname == 'pyglet':
             # noinspection PyProtectedMember
             monkey_patch_pyglet(MockTurtle._screen.cv)
+        elif self.fullname == 'PIL.ImageShow':
+            module.show = partial(self.mock_show_pillow)
 
-    def mock_show(self, *_args, **_kwargs):
+    # noinspection PyUnusedLocal
+    def mock_show_pillow(self, image, title=None, **options):
+        position = self._center_position(*image.size)
+        LivePillowImage(image).display(position)
+
+    def mock_show_matplotlib(self, *_args, **_kwargs):
         figure = self.plt.gcf()
+        figure_width = figure.get_figwidth() * figure.dpi
+        figure_height = figure.get_figheight() * figure.dpi
+        position = self._center_position(figure_width, figure_height)
+        LiveFigure(figure).display(position)
+
+    def live_coding_zoom_matplotlib(self):
+        screen_width, screen_height = self.plt.live_coding_size
+        fig = self.plt.gcf()
+        fig_width, fig_height = fig.get_figwidth(), fig.get_figheight()
+        x_dpi = screen_width/fig_width
+        y_dpi = screen_height/fig_height
+        fig.dpi = min(x_dpi, y_dpi)
+
+    @staticmethod
+    def _screen_size():
         # noinspection PyProtectedMember
         turtle_screen = MockTurtle._screen
         screen_width = turtle_screen.cv.cget('width')
         screen_height = turtle_screen.cv.cget('height')
-        figure_width = figure.get_figwidth()*figure.dpi
-        figure_height = figure.get_figheight()*figure.dpi
+        return screen_width, screen_height
+
+    def _center_position(self, figure_width, figure_height):
+        screen_width, screen_height = self._screen_size()
         if figure_width < screen_width:
             x = (screen_width - figure_width) // 2
         else:
@@ -437,13 +468,4 @@ class PatchedModuleLoader(Loader):
         # Adjust to turtle coordinates.
         x -= screen_width // 2
         y = screen_height // 2 - y
-
-        LiveFigure(figure).display((x, y))
-
-    def live_coding_zoom(self):
-        screen_width, screen_height = self.plt.live_coding_size
-        fig = self.plt.gcf()
-        fig_width, fig_height = fig.get_figwidth(), fig.get_figheight()
-        x_dpi = screen_width/fig_width
-        y_dpi = screen_height/fig_height
-        fig.dpi = min(x_dpi, y_dpi)
+        return x, y
